---------------------------------------------------------------------------------
-- SavedVariables
---------------------------------------------------------------------------------
Dra2_Config		= {};
Dra2_Config.GuiShow	= true;
Dra2_Config.Maximized	= true;
Dra2_Config.Scale	= 100;


---------------------------------------------------------------------------------
-- LocalVariables
---------------------------------------------------------------------------------
Dra2			= {};
Dra2.Version		= "2.12.5";
Dra2.PlayerName		= UnitName ("player");
Dra2.PlayerDruid	= nil;
Dra2.RebirthID		= 0;
Dra2.InnervateID	= 0;
Dra2.UpdateInterval	= 0.2; -- seconds
Dra2.UpdateTimer	= 0.0;
Dra2.Ticks		= 0;
Dra2.TicksInterval	= 3;
Dra2.AddonChannel	= "RAID";
Dra2.CheckSpells	= nil;
Dra2.CheckMembers	= nil;
Dra2.AskForSpells	= nil;
Dra2.DruidsData		= {};


---------------------------------------------------------------------------------
-- OnLoad
---------------------------------------------------------------------------------
function Dra2.OnLoad ()

	Dra2.Print ("Dra"..Dra2.Version.." Loaded");
	
	this:RegisterEvent ("CHAT_MSG_ADDON");
	this:RegisterEvent ("PARTY_MEMBERS_CHANGED");
	this:RegisterEvent ("PLAYER_ENTERING_WORLD");
	this:RegisterEvent ("SPELLCAST_STOP");
	this:RegisterEvent ("SPELLS_CHANGED");
	this:RegisterEvent ("VARIABLES_LOADED");
	
	SLASH_DRATWOSLASH1 = "/dra2";
	SlashCmdList["DRATWOSLASH"] = Dra2.CmdHandler;
	
end


---------------------------------------------------------------------------------
-- Command Line Handler
---------------------------------------------------------------------------------
function Dra2.CmdHandler (msg)

	msg = string.lower (msg);
	
	if (msg == "gui") then -- toggle gui
		if (Dra2_Config.GuiShow) then
			Dra2.Print ("Window hidden");
			Dra2_GUI_Frame:Hide ();
			Dra2_Config.GuiShow = false;
		else
			Dra2.Print ("Window shown");
			Dra2_GUI_Frame:Show ();
			Dra2_Config.GuiShow = true;
		end
		Dra2.GuiSizeToggle ();
		
	elseif (msg == "scale") then -- get scale
		Dra2.Print ("Window scale set to "..Dra2_Config.Scale.."%");
		
	elseif (string.sub (msg, 1, 6) == "scale ") then -- scale window
		local scale = tonumber (string.sub (msg, 7));
		scale = math.max (math.min (scale, 150), 50);
		Dra2_Config.Scale = scale;
		Dra2_GUI_Frame:SetScale (Dra2_Config.Scale/100);
		Dra2.Print ("Window scale set to "..Dra2_Config.Scale.."%");
		
	elseif (msg == "check") then -- ask for cooldowns and check members
		Dra2.GetSpellsID ();
		Dra2.CheckMembers = true;
		Dra2.CheckSpells = true;
		Dra2.AskForSpells = true;
		Dra2.Print ("Checking members and cooldowns");
		
	elseif (msg == "help") then
		Dra2.Help ();
		
	else
		Dra2.Print ("Unknown command |cfffff000/dra2 "..msg.."|r");
		
	end
end


---------------------------------------------------------------------------------
-- OnUpdate
---------------------------------------------------------------------------------
function Dra2.OnUpdate ()

	Dra2.UpdateTimer = Dra2.UpdateTimer + arg1;
	if (Dra2.UpdateTimer > Dra2.UpdateInterval) then
		Dra2.UpdateTimer = 0.0;
		
		if (Dra2.CheckSpells) then
			Dra2.Ticks = Dra2.Ticks + 1;
			if (Dra2.Ticks == Dra2.TicksInterval) then				
				--if (Dra2.CheckMembers) then
				--	Dra2.CheckMembers = nil;
				--	Dra2.MembersChanged ();
				--	Dra2.GuiSizeToggle ();
				--end
				
				if (Dra2.AskForSpells) then
					Dra2.AskForSpells = nil;
					SendAddonMessage ("DRA2_ASK", "ASK", Dra2.AddonChannel);
				end
				
			end
			if (Dra2.Ticks > Dra2.TicksInterval) then
				Dra2.CheckSpells = nil;
				Dra2.Ticks = 0;
				Dra2.SendCooldowns ();
			end
		end
		
		if (Dra2.CheckMembers) then
			Dra2.CheckMembers = nil;
			Dra2.MembersChanged ();
			Dra2.GuiSizeToggle ();
		end
		
		Dra2.UpdateCounters ();
	end
	
end


---------------------------------------------------------------------------------
-- Send spell cooldowns via SendAddonMessage
---------------------------------------------------------------------------------
function Dra2.SendCooldowns ()
	
	if (Dra2.PlayerDruid) then
		local cooldown = Dra2.GetCooldown (Dra2.RebirthID).."#"..Dra2.GetCooldown (Dra2.InnervateID);
		SendAddonMessage("DRA2", cooldown, Dra2.AddonChannel);
	end
	
end


---------------------------------------------------------------------------------
-- Update cooldown counters
---------------------------------------------------------------------------------
function Dra2.UpdateCounters ()
	
	local i;
	local names = {};
	local rebcd = {};
	local inncd = {};
	local rebirth = 0;
	local innerv = 0;
	local membername = "";	
	local prefix = "";
	
	for i = 1,40 do
		if (Dra2.DruidsData[i]) then
			prefix = "";
			if (not UnitIsConnected (Dra2.DruidsData[i].frame)) then -- unit is disconnected
				prefix = "|cffff3333*DC*|r";
				Dra2.DruidsData[i].addon = nil;
			elseif (UnitIsDeadOrGhost (Dra2.DruidsData[i].frame)) then -- unit is dead (or ghost)
				prefix = "|cffff3333*RIP*|r";
			end
			
			if (Dra2.DruidsData[i].addon) then
				membername = prefix..Dra2.DruidsData[i].name;
			else
				membername = prefix.."|cff999999"..Dra2.DruidsData[i].name.."|r";
			end
			
			if (Dra2.DruidsData[i].rebirth < 0) then
				rebirth = "--------";
			else
				rebirth = Dra2.SecondsToMinutes (math.max (math.ceil (Dra2.DruidsData[i].rebirth - (GetTime () - Dra2.DruidsData[i].time)), 0));
			end
			
			if (Dra2.DruidsData[i].innervate < 0) then
				innerv = "--------";
			else
				innerv = Dra2.SecondsToMinutes (math.max (math.ceil (Dra2.DruidsData[i].innervate - (GetTime () - Dra2.DruidsData[i].time)), 0));
			end
			table.insert (names, membername);
			table.insert (rebcd, rebirth);
			table.insert (inncd, innerv);
		else
			break;
		end
	end
	
	Dra2_GUI_Frame_TextName:SetText (table.concat (names, "\n"));
	Dra2_GUI_Frame_TextRebirth:SetText (table.concat (rebcd, "\n"));
	Dra2_GUI_Frame_TextInnerv:SetText (table.concat (inncd, "\n"));
	
end


---------------------------------------------------------------------------------
-- Event Handler
---------------------------------------------------------------------------------
function Dra2.OnEvent ()

	-- Player members changed
	if (event == "PARTY_MEMBERS_CHANGED") then
		Dra2.CheckMembers = true;
		Dra2.CheckSpells = true;
		
	-- Spellcast was finished
	elseif (event == "SPELLCAST_STOP") then
		Dra2.CheckSpells = true;
		
	-- Received message from addon
	elseif ((event == "CHAT_MSG_ADDON") and (arg1 == "DRA2")) then
		local i;
		for i = 1,40 do
			if (Dra2.DruidsData[i]) then
				if (Dra2.DruidsData[i].name == arg4) then
					local pos, _ = string.find(arg2, "#");
					local rcd = tonumber (string.sub(arg2, 1, pos-1));
					local icd = tonumber (string.sub(arg2, pos+1));
					Dra2.DruidsData[i].time	= GetTime ();
					Dra2.DruidsData[i].rebirth = rcd;
					Dra2.DruidsData[i].innervate = icd;
					Dra2.DruidsData[i].addon = true;
					break;
				end
			end
		end
		
	-- Received ASK command
	elseif ((event == "CHAT_MSG_ADDON") and (arg1 == "DRA2_ASK") and (arg4 ~= Dra2.PlayerName)) then
		Dra2.CheckSpells = true;
		
	-- Get rebirth and innervate spellbook IDs
	elseif (event == "SPELLS_CHANGED") then
		Dra2.GetSpellsID ();
		
	elseif (event == "PLAYER_ENTERING_WORLD") then
		Dra2.GetSpellsID ();
		Dra2.CheckMembers = true;
		Dra2.CheckSpells = true;
		Dra2.AskForSpells = true;
		
	-- Veriables loaded - check if they are OK
	elseif (event == "VARIABLES_LOADED") then
		if (Dra2_Config.GuiShow == nil) then
			Dra2_Config.GuiShow = true;
		end
		if (Dra2_Config.Maximized == nil) then
			Dra2_Config.Maximized = true;
		end
		if (Dra2_Config.Scale == nil) then
			Dra2_Config.Scale = 100;
		end
		
		Dra2_GUI_Frame:SetScale (Dra2_Config.Scale/100);
		
		if (Dra2_Config.GuiShow) then
			Dra2_GUI_Frame:Show ();
			Dra2.GuiSizeToggle ();
		end
		
		_, class = UnitClass ("player");
		if (class == "DRUID") then
			Dra2.PlayerDruid = true;
		end
	end

end


---------------------------------------------------------------------------------
-- Party members changed
---------------------------------------------------------------------------------
function Dra2.MembersChanged ()
	
	local i;
	local class;
	local num = 1;
	local membername;
	
	local t_time = {};
	local t_rebi = {};
	local t_inne = {};
	local t_addo = {};
	
	for i = 1,40 do
		if (Dra2.DruidsData[i]) then
			t_time[Dra2.DruidsData[i].name] = Dra2.DruidsData[i].time;
			t_rebi[Dra2.DruidsData[i].name] = Dra2.DruidsData[i].rebirth;
			t_inne[Dra2.DruidsData[i].name] = Dra2.DruidsData[i].innervate;
			t_addo[Dra2.DruidsData[i].name] = Dra2.DruidsData[i].addon;
		else
			break;
		end
	end
	
	Dra2.DruidsData = {};
	
	-- Raid
	if (GetNumRaidMembers() > 0) then
		for i = 1,40 do
			_, class = UnitClass ("raid"..i);
			if (class == "DRUID") then
				membername = UnitName ("raid"..i);
				if (t_time[membername]) then
					Dra2.DruidsData[num] = {
						name		= membername;
						time		= t_time[membername];
						rebirth		= t_rebi[membername];
						innervate	= t_inne[membername];
						addon		= t_addo[membername];
						frame		= "raid"..i;
					}
				else
					Dra2.DruidsData[num] = {
						name		= membername;
						time		= GetTime ();
						rebirth		= -1;
						innervate	= -1;
						addon		= nil;
						frame		= "raid"..i;
					}
				end
				num = num + 1;
			end
		end
		
	-- Party
	elseif (GetNumPartyMembers() > 0) then
		_, class = UnitClass ("player");
		if (class == "DRUID") then
			membername = UnitName ("player");
			if (t_time[membername]) then
				Dra2.DruidsData[num] = {
					name		= membername;
					time		= t_time[membername];
					rebirth		= t_rebi[membername];
					innervate	= t_inne[membername];
					addon		= t_addo[membername];
					frame		= "player";
				}
			else
				Dra2.DruidsData[num] = {
					name		= membername;
					time		= GetTime ();
					rebirth		= -1;
					innervate	= -1;
					addon		= nil;
					frame		= "player";
				}
			end
			num = num + 1;
		end
		for i = 1,4 do
			_, class = UnitClass ("party"..i);
			if (class == "DRUID") then
				membername = UnitName ("party"..i);
				if (t_time[membername]) then
					Dra2.DruidsData[num] = {
						name		= membername;
						time		= t_time[membername];
						rebirth		= t_rebi[membername];
						innervate	= t_inne[membername];
						addon		= t_addo[membername];
						frame		= "party"..i;
					}
				else
					Dra2.DruidsData[num] = {
						name		= membername;
						time		= GetTime ();
						rebirth		= -1;
						innervate	= -1;
						addon		= nil;
						frame		= "party"..i;
					}
				end
				num = num + 1;
			end
		end
		
	end
end


---------------------------------------------------------------------------------
-- Gui size toggle
---------------------------------------------------------------------------------
function Dra2.GuiSizeToggle ()

	if (Dra2_Config.Maximized) then -- frame maximized
		Dra2_GUI_FrameMaximize:Hide();
		Dra2_GUI_FrameMinimize:Show();
		Dra2_GUI_Frame_HeaderName:Show();
		Dra2_GUI_Frame_HeaderInnerv:Show();
		Dra2_GUI_Frame_HeaderRebirth:Show();
		Dra2_GUI_Frame_TextName:Show();
		Dra2_GUI_Frame_TextInnerv:Show();
		Dra2_GUI_Frame_TextRebirth:Show();
		-- Get font height
		local _, fontHeight, _ = GameFontNormalSmall:GetFont();
		local height = math.ceil (table.getn (Dra2.DruidsData)*fontHeight + 40);
		-- Set gui frame height
		Dra2_GUI_Frame:SetHeight(height);
		
	else -- frame minimized
		Dra2_GUI_FrameMaximize:Show();
		Dra2_GUI_FrameMinimize:Hide();
		Dra2_GUI_Frame_HeaderName:Hide();
		Dra2_GUI_Frame_HeaderInnerv:Hide();
		Dra2_GUI_Frame_HeaderRebirth:Hide();
		Dra2_GUI_Frame_TextName:Hide();
		Dra2_GUI_Frame_TextInnerv:Hide();
		Dra2_GUI_Frame_TextRebirth:Hide();
		-- Set gui frame height
		Dra2_GUI_Frame:SetHeight(23);
	end
	
end


---------------------------------------------------------------------------------
-- Get Rebirth and Innervate IDs
---------------------------------------------------------------------------------
function Dra2.GetSpellsID ()

	if (Dra2.PlayerDruid) then
		Dra2.RebirthID		= 0;
		Dra2.InnervateID	= 0;
		
		local i=1;
		while true do
			local SpellName, _ = GetSpellName (i, BOOKTYPE_SPELL);
			if (not SpellName) then
				break;
			end
			if (SpellName == Dra2_Strings.SpellRebirth) then
				Dra2.RebirthID = i;
			end
			if (SpellName == Dra2_Strings.SpellInnervate) then
				Dra2.InnervateID = i;
			end
			if ((Dra2.RebirthID > 0) and (Dra2.InnervateID > 0)) then
				break;
			end
			i = i + 1;
		end
	end
	
end


---------------------------------------------------------------------------------
-- Print
---------------------------------------------------------------------------------
function Dra2.Print (msg)

	DEFAULT_CHAT_FRAME:AddMessage("Dra2 >> "..msg, 1.0, 0.70, 0.20);
	
end


---------------------------------------------------------------------------------
-- Get spell cooldown
---------------------------------------------------------------------------------
function Dra2.GetCooldown (id)

	if (Dra2.PlayerDruid) then
		if (id == 0) then
			return -1;
		end
		
		local cooldown = 0;
		local start, duration = GetSpellCooldown (id, BOOKTYPE_SPELL);
		
		if (start > 0 and duration > 0) then
			cooldown = math.ceil (duration - (GetTime() - start));
		end
		if (cooldown < 3) then
			cooldown = 0;
		end
		
		if (cooldown > 3600) then
			cooldown = 0;
		end
		
		return cooldown;
	else
		return 0;
	end

end


---------------------------------------------------------------------------------
-- Seconds to minutes:seconds
---------------------------------------------------------------------------------
function Dra2.SecondsToMinutes (seconds)

	local min = math.floor (seconds/60);
	local sec = math.mod (seconds, 60);
	
	if (min == 0 and sec == 0) then
		return "|cff00ff00Ready|r";
	end
	
	local retval = "";
	
	if (min < 10) then
		retval = retval.."0";
	end
	retval = retval..min..":";
	if (sec < 10) then
		retval = retval.."0";
	end
	retval = retval..sec
	
	if (min < 3) then
		retval = "|cffffff00"..retval.."|r";
	else
		retval = "|cffff0000"..retval.."|r";
	end
	return retval;
	
end


---------------------------------------------------------------------------------
-- Help
---------------------------------------------------------------------------------
function Dra2.Help ()

	Dra2.Print ("== Help ==");
	Dra2.Print ("|cffffff00/dra2 gui|r - show/hide DRA2 window");
	Dra2.Print ("|cffffff00/dra2 scale|r - check window scale");
	Dra2.Print ("|cffffff00/dra2 scale [50-150]|r - set window scale from 50% to 150%");
	Dra2.Print ("|cffffff00/dra2 check|r - check for raid member changes and ask for cooldowns");
	Dra2.Print ("|cffffff00/dra2 help|r - this help");

end
